package com.yizhi.application.controller.api;


import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.application.domain.TrSignRecord;
import com.yizhi.application.export.ExportSignRecord;
import com.yizhi.application.service.SignRecordService;
import com.yizhi.util.application.domain.Response;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 签到记录表 前端控制器 学员端
 * </p>
 *
 * @author fanchunhui123
 * @since 2018-03-07
 */
@Api(tags = "签到-学员端", description = "签到记录表学员端接口")
@RestController
@RequestMapping("/api/sign/record")
public class SignRecordController {
    Logger logger = LoggerFactory.getLogger(SignRecordController.class);

    @Autowired
    SignRecordService signRecordService;

    @Autowired
    ExportSignRecord exportSignRecord;

    /**
     * 签到记录信息
     *
     * @param trainingProjectId
     * @return
     */
    @PostMapping("/list")
    public Response<String> list(
            @ApiParam(value = "培训项目ID", required = true) String trainingProjectId) {
        return Response.ok();
    }

    /**
     * 我的签到总数量
     *
     * @param accountId 学员ID
     * @return
     */
    @GetMapping("/count")
    public Integer selectSignRecordCountByAccountId(@RequestParam("accountId") Long accountId) {
        Integer count = signRecordService.selectCountByAccountId(accountId);
        if (null == count) {
            count = 0;
        }

        return count;
    }


    /**
     * 获取签到完成数量( PC 端  学习统计签到 完成数量)
     *
     * @param trainingProjectId 培训项目ID
     * @return
     */
    @GetMapping("/pc/finishedCount")
    public Integer selectCountByTrainingProjectId(@RequestParam("trainingProjectId") Long trainingProjectId, @RequestParam("accountId") Long accountId) {
        logger.info("trainingProjectId为：" + trainingProjectId + "accountId:" + accountId);
        Integer count = signRecordService.selectCountByTrainingProjectId(trainingProjectId, accountId);
        if (null == count) {
            count = 0;
        }
        logger.info("已完成的签到数量为：" + count);
        return count;
    }

    /**
     * 签到记录导出
     *
     * @param trainingProjectId
     * @return
     */
    @GetMapping("/export")
    public String exportSignRecord(@RequestParam(name = "trainingProjectId") Long trainingProjectId) {
        Map<String, Object> map = new HashMap<>();
        RequestContext res = ContextHolder.get();
        Long taskId = System.currentTimeMillis();
        String serialNo = "SIGN-RECORD-EXPORT" + taskId.toString();
        String taskName = "签到记录导出" + taskId.toString();
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("res", res);
        map.put("trainingProjectId", trainingProjectId);
        exportSignRecord.execute(map, true);
        return serialNo;
    }
    
    @PostMapping("/insert")
    public boolean insertSignRecordList(@RequestBody List<TrSignRecord> signRecordList) {
		return signRecordService.insertSignRecordList(signRecordList);
    }
}

