package com.yizhi.application.controller.remote;

import com.yizhi.application.domain.TrSign;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.service.SignService;
import com.yizhi.util.application.domain.Response;
import com.yizhi.sign.application.vo.SignIdVO;
import com.yizhi.sign.application.vo.SignQrcodeVO;
import com.yizhi.sign.application.vo.SignVO;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 签到主体表，入口在培训项目 前端控制器 管理端
 * </p>
 *
 * @author fanchunhui123
 * @since 2018-03-07
 */
@RestController
@RequestMapping("/remote/manage/sign")
public class RemoteSignManageController {

	private static final Logger LOG = LoggerFactory.getLogger(RemoteSignManageController.class);

	@Autowired
	SignService signService;
	
	@Autowired
    IdGenerator idGenerator;

	@PostMapping(value = "/insert")
	public Long insert(@ApiParam(value = "签到主体表", required = true) @RequestBody SignVO signVO) {
		try {
			Long id = idGenerator.generate();
			signVO.setId(id);
			boolean result = signService.insertSignAndSignTimeAndSignRemind(signVO);
			if(!result){
				id = null;
			}
			return id;
		} catch (Exception e) {
			LOG.error("新建签到失败！", e);
			return null;
		}
	}

	@PostMapping(value = "/view")
	public SignVO getSign(@ApiParam(value = "培训项目id", required = true) @RequestBody SignIdVO signIdVO) {
		SignVO signVO = null;
		try {
			signVO = signService.querySign(signIdVO.getId());
			return signVO;
		} catch (Exception e) {
			LOG.error("新建签到失败！", e);
			return signVO;
		}
	}

	@PostMapping("/update")
	public boolean update(@ApiParam(value = "签到主体表", required = true) @RequestBody SignVO signVO) {
		try {
			boolean result = signService.updateSignAndSignTimeAndSignRemind(signVO);
			return result;
		} catch (Exception e) {
			LOG.error("编辑签到失败！", e);
			return false;
		}
	}

	@PostMapping("/report")
	public Response<String> report() {
		return Response.ok();
	}

	@PostMapping("/report/export")
	public Response<String> reportExport(@ApiParam(value = "培训项目ID", required = true) String trainingProjectId) {
		return Response.ok();
	}

	@PostMapping("/delete")
	public boolean deleteByProjectId(@RequestBody SignIdVO signIdVO) {
		return signService.deleteSignByProjectId(signIdVO);
	}
	
	/**
	 * 获取生成的二维码Url
	 * 
	 * @param signIdVO
	 * @return
	 */
	@PostMapping("/qrcode/get")
	public List<SignQrcodeVO> qrcode(@RequestBody SignIdVO signIdVO) {
		try {
			// 校验培训项目id
			if (StringUtils.isBlank(String.valueOf(signIdVO.getId()))) {
				return null;
			}

			// TODO 目前没有项目
			// 查询生成二维码必需参数
			List<SignQrcodeVO> resultList = signService.querySignTime(signIdVO.getId());
			LOG.info("成功获取生成二维码必需数据");
 
			return resultList;

		} catch (Exception e) {
			LOG.error("", e);
			return null;
		}
	}
	@ApiOperation("根据签到id获取签到")
	@GetMapping("/one/get")
	public com.yizhi.sign.application.vo.domain.TrSign getSign(
			@RequestParam(name = "signId" )Long signId
	){
		TrSign trSign = signService.selectById(signId);
		com.yizhi.sign.application.vo.domain.TrSign trSign1 = new com.yizhi.sign.application.vo.domain.TrSign();
		BeanUtils.copyProperties(trSign,trSign1);
		return trSign1;
	}

	@ApiOperation("根据签到时间段id获取签到")
	@GetMapping("/time/get")
	public  com.yizhi.sign.application.vo.domain.TrSign getSignByTime(@RequestParam(name = "signTimeId" )Long signId){
		TrSign trSign = signService.selectByTimeId(signId);
		com.yizhi.sign.application.vo.domain.TrSign trSign1 = new com.yizhi.sign.application.vo.domain.TrSign();
		BeanUtils.copyProperties(trSign,trSign1);
		return trSign1;
	}
}
