package com.yizhi.application.export;

import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.application.service.SignRecordService;
import com.yizhi.sign.application.vo.ExportSignRecordVO;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Auther: ding
 * @Date: 2019/06/04
 * @Description:按考试统计查看结果下载
 */
@Component
public class ExportSignRecord extends AbstractDefaultTask<String, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExportSignRecord.class);


    @Autowired
    private SignRecordService signRecordService;

    @Override
    protected String execute(Map<String, Object> map) {
        String uploadUrl = null;


        Long trainingProjectId = (Long) map.get("trainingProjectId");
        RequestContext res = (RequestContext) map.get("res");

        Long accountId = res.getAccountId();
        Long siteId = res.getSiteId();
        Long companyId = res.getCompanyId();
        boolean isAdmin = res.isAdmin();


        Date comitTime = new Date();
        Long taskId = (Long) map.get("taskId");
        String serialNo = (String) map.get("serialNo");
        String taskName = (String) map.get("taskName");
        SimpleDateFormat sdf2 = new SimpleDateFormat("yyyyMMddHHmmss");
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, accountId, comitTime, siteId, companyId);
        working(taskContext);

        List<ExportSignRecordVO> list = null;

        try {
            list = signRecordService.exportSignRecord(accountId, isAdmin, siteId, trainingProjectId);
        } catch (Exception e) {
            LOGGER.info("签到记录明细获取数据过程中出现错误", e);
            fail(taskContext, "签到记录明细获取数据过程中出现错误");
        }


        try {

            if (CollectionUtils.isEmpty(list)) {
                success(taskContext, "未找到数据", "");
            }

            String traningProjectName = list.get(0).getTainingProjectName();

            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");

            XSSFWorkbook wb = new XSSFWorkbook();
            XSSFSheet sheet = wb.createSheet("签到记录明细表");
            XSSFRow row = sheet.createRow(0);

//            HSSFCellStyle cellStyle = wb.createCellStyle();
//            //水平居中
//            cellStyle.setAlignment(HorizontalAlignment.CENTER);
//            //垂直居中
//            cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);

//            HSSFFont font = wb.createFont();
//            //字体加粗
//            font.setBold(true);
//            //字体大小14
//            font.setFontHeightInPoints((short) 14);
//
//            cellStyle.setFont(font);


            CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 12);
            sheet.addMergedRegion(cra);
            XSSFCell cell = row.createCell(0);
//            cell.setCellStyle(cellStyle);
            cell.setCellValue(traningProjectName + "签到记录明细表");

            row = sheet.createRow(1);
            cra = new CellRangeAddress(1, 1, 0, 12);
            sheet.addMergedRegion(cra);
            cell = row.createCell(0);
            cell.setCellValue("统计周期：截止：" + sdf.format(new Date()));

            XSSFRow row2 = sheet.createRow(2);
            row2.createCell(0).setCellValue("项目名称");
            row2.createCell(1).setCellValue("签到时间段");
            row2.createCell(2).setCellValue("用户名");
            row2.createCell(3).setCellValue("姓名");
            row2.createCell(4).setCellValue("签到状态");
            row2.createCell(5).setCellValue("签到时间");
            row2.createCell(6).setCellValue("所在部门");
            row2.createCell(7).setCellValue("用户状态");

            if (CollectionUtils.isNotEmpty(list)) {
                for (ExportSignRecordVO recordVO : list) {
                    row = sheet.createRow(list.indexOf(recordVO) + 3);
                    row.createCell(0).setCellValue(recordVO.getTainingProjectName() == null ? "--" : recordVO.getTainingProjectName());
                    row.createCell(1).setCellValue(sdf.format(recordVO.getStartTime()) + "~" + sdf.format(recordVO.getEndTime()));
                    row.createCell(2).setCellValue(recordVO.getAccountName() == null ? "--" : recordVO.getAccountName());
                    row.createCell(3).setCellValue(recordVO.getAccountFullName() == null ? "--" : recordVO.getAccountFullName());
                    if (recordVO.getSignState() == 0) {
                        row.createCell(4).setCellValue("未签到");
                    } else if (recordVO.getSignState() == 1) {
                        row.createCell(4).setCellValue("已签到");
                    } else if (recordVO.getSignState() == 2) {
                        row.createCell(4).setCellValue("已补签");
                    }
                    row.createCell(5).setCellValue(recordVO.getSignTime() == null ? "--" : sdf.format(recordVO.getSignTime()));
                    row.createCell(6).setCellValue(recordVO.getOrgName() == null ? "--" : recordVO.getOrgName());
                    row.createCell(7).setCellValue(recordVO.getAccountState() == 0 ? "停用" : "启用");
                }
            }


            String fileName = traningProjectName + "签到记录明细表" + sdf2.format(comitTime) + ".xlsx";
//            String requestPath = "/Users/dingxiaowei/aaa";
            String requestPath = FileConstant.SAVE_PATH;
            File fileDir = new File(requestPath);
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }
            String path = requestPath + "/" + fileName;
            FileOutputStream os = null;
            File file = null;

            try {
                os = new FileOutputStream(path);
                wb.write(os);
                uploadUrl = OssUpload.upload(path, fileName);
                file = new File(path);
                success(taskContext, "成功", uploadUrl);
            } catch (Exception e2) {
                e2.printStackTrace();
                fail(taskContext, "签到记录明细表,写入过程出现错误");
                LOGGER.error("签到记录明细表,写入过程出现错误");
            } finally {
                if (os != null) {
                    os.close();
                }
                if (wb != null) {
                    wb.close();
                }
                if (file != null) {
                    file.delete();
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            fail(taskContext, "签到记录明细表,下载考试结果数据出现错误");
            LOGGER.error("签到记录明细表,下载考试结果数据出现错误");
        }

        return uploadUrl;
    }
}
