package com.yizhi.application.mapper;

import java.util.List;

import com.yizhi.application.domain.TrSignTime;
import com.yizhi.application.domain.TrSignRecord;
import com.yizhi.sign.application.vo.ExportSignRecordVO;
import com.yizhi.training.application.vo.domain.TpAuthorizationRangeVo;
import com.yizhi.training.application.vo.domain.TrainingProjectVo;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.sign.application.vo.TrSignRecordVo;

/**
 * <p>
 * 签到记录表 Mapper 接口
 * </p>
 *
 * @author fanchunhui123
 * @since 2018-03-07
 */
public interface SignRecordMapper extends BaseMapper<TrSignRecord> {

	/**
	 * 获取我的签到
	 * @param accountId 学员ID
	 * @param month，月份可为空查询所有
	 * @return
	 */
	List<TrSignRecordVo> selectSignRecordListByAccountIdAndMonth(@Param("page") Page<TrSignRecordVo> page, @Param("accountId") Long accountId,
			@Param("year") String year, @Param("month") String month);

	@Select("SELECT COUNT(id) from tr_sign_record where account_id = #{accountId}")
	Integer selectCountByAccountId(@Param("accountId")Long accountId);


	/**
	 * 获取签到完成数量( PC 端  学习统计签到 完成数量)
	 * @param trainingProjectId
	 * @return
	 */
    @Select("SELECT COUNT(id) from tr_sign_record where training_project_id = #{trainingProjectId} and account_id = #{accountId}")
    Integer selectCountByTrainingProjectId(@Param("trainingProjectId") Long trainingProjectId, @Param("accountId") Long accountId);

	/**
	 * 查找培训项目
	 * @param trainingProjectId
	 * @return
	 */
	TrainingProjectVo getTainningProjectById(@Param("trainingProjectId") Long trainingProjectId);

	/**
	 * 查找指定范围用户或者部门id
	 * @param trainingProjectId
	 * @return
	 */
	List<TpAuthorizationRangeVo> getAuthorizations(@Param("trainingProjectId") Long trainingProjectId);

	/**
	 * 查找培训项目下所有签到时间段
	 * @param trainingProjectId
	 * @return
	 */
	List<TrSignTime> selectSignTimeList(@Param("trainingProjectId") Long trainingProjectId);

	/**
	 * 获取某一时间段的签到记录
	 * @param signTimeId
	 * @param accountIds
	 * @return
	 */
	List<ExportSignRecordVO> exportSignRecord(@Param("signTimeId") Long signTimeId,@Param("accountIds") List<Long> accountIds);
}
