package com.yizhi.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.system.application.constant.AuthzConstant;
import com.yizhi.application.domain.TrSignRecord;
import com.yizhi.application.domain.TrSignTime;
import com.yizhi.application.mapper.SignRecordMapper;
import com.yizhi.application.mapper.SignTimeMapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.service.SignRecordService;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.system.application.vo.HQAccountInManageParam;
import com.yizhi.system.application.vo.ReportAccountRespInManageVO;
import com.yizhi.system.application.vo.ReportAccountRespVO;
import com.yizhi.system.application.vo.ReportRangeAccountReqVO;
import com.yizhi.util.application.date.DateUtil;
import com.yizhi.training.application.vo.domain.TpAuthorizationRangeVo;
import com.yizhi.training.application.vo.domain.TrainingProjectVo;
import com.yizhi.sign.application.vo.ExportSignRecordVO;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 签到记录表 服务实现类
 * </p>
 *
 * @author fanchunhui123
 * @since 2018-03-07
 */
@Service
public class SignRecordServiceImpl extends ServiceImpl<SignRecordMapper, TrSignRecord> implements SignRecordService {

    @Autowired
    SignRecordMapper signRecordMapper;

    @Autowired
    ReportClient reportClient;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private SignTimeMapper signTimeMapper;

    /**
     * 我的签到总数量
     *
     * @param accountId 学员ID
     * @return
     */
    @Override
    public Integer selectCountByAccountId(Long accountId) {
        return signRecordMapper.selectCountByAccountId(accountId);
    }


    /**
     * 获取签到完成数量( PC 端  学习统计签到 完成数量)
     *
     * @param trainingProjectId 培训项目ID
     * @return
     */
    @Override
    public Integer selectCountByTrainingProjectId(Long trainingProjectId, Long accountId) {
        return signRecordMapper.selectCountByTrainingProjectId(trainingProjectId, accountId);
    }

    @Override
    public List<ExportSignRecordVO> exportSignRecord(Long accountId, boolean isAdmin, Long siteId, Long trainingProjectId) {
        List<Long> accountIds = new ArrayList<>();
        Map<Long, ReportAccountRespVO> accountMap = new HashMap<>();

        TrainingProjectVo trainingProject = signRecordMapper.getTainningProjectById(trainingProjectId);
        List<Long> relationAccountIds = new ArrayList<>();
        List<Long> relationOrgIds = new ArrayList<>();
        //如果是指定范围找出指定范围的部门或者用户id
        if (trainingProject.getVisibleRange() == 0) {

            List<TpAuthorizationRangeVo> tpAuthorizationRanges = signRecordMapper.getAuthorizations(trainingProjectId);
            if (CollectionUtils.isNotEmpty(tpAuthorizationRanges)) {
                for (TpAuthorizationRangeVo tpAuthorizationRange : tpAuthorizationRanges) {
                    //1是指定部门
                    if (tpAuthorizationRange.getType() == 1) {
                        relationOrgIds.add(tpAuthorizationRange.getRelationId());
                    } else if (tpAuthorizationRange.getType() == 2) {
                        //2是指定用户
                        relationAccountIds.add(tpAuthorizationRange.getRelationId());
                    }
                }
            } else {
                return null;
            }
        }
        if (null == trainingProject) {
            return null;
        }
        boolean isUseAccountIds = false;
        if (isAdmin) {
            //管理员
            ReportRangeAccountReqVO reqVO = new ReportRangeAccountReqVO();
            //如果是指定范围放入指定的部门或者用户id
            if (trainingProject.getVisibleRange() == 0) {
                isUseAccountIds = true;
                if (CollectionUtils.isNotEmpty(relationAccountIds)) {
                    reqVO.setAccountIds(relationAccountIds);
                }
                if (CollectionUtils.isNotEmpty(relationOrgIds)) {
                    reqVO.setOrgIds(relationOrgIds);
                }
            }
            reqVO.setSiteId(siteId);
            List<ReportAccountRespVO> accounts = reportClient.getRangeAccounts(reqVO);
            //将用户id拿出并将用户信息放入map中
            if (CollectionUtils.isNotEmpty(accounts)) {
                for (ReportAccountRespVO accountRespVO : accounts) {
                    accountIds.add(accountRespVO.getUserId());
                    accountMap.put(accountRespVO.getUserId(), accountRespVO);
                }
            } else {
                return null;
            }
        } else {
            isUseAccountIds = true;
            List<Long> relationIds = null;
            //一般管理员
            if (trainingProject.getVisibleRange() == 0) {
                ReportRangeAccountReqVO reqVO = new ReportRangeAccountReqVO();
                if (CollectionUtils.isNotEmpty(relationAccountIds)) {
                    reqVO.setAccountIds(relationAccountIds);
                }
                if (CollectionUtils.isNotEmpty(relationOrgIds)) {
                    reqVO.setOrgIds(relationOrgIds);
                }
                reqVO.setSiteId(siteId);
                List<ReportAccountRespVO> accounts = reportClient.getRangeAccounts(reqVO);
                if (CollectionUtils.isNotEmpty(accounts)) {
                    relationIds = accounts.stream().map(ReportAccountRespVO::getUserId).collect(Collectors.toList());
                } else {
                    return null;
                }
            }
            //查找出管辖区下所有的用户
            HQAccountInManageParam reqVO = new HQAccountInManageParam();
            reqVO.setAccountId(accountId);
            reqVO.setSiteId(siteId);
            reqVO.setModuleType(AuthzConstant.moduleType.trainingProject);
            ReportAccountRespInManageVO respInManageVO = reportClient.getRangeAccountsInManage(reqVO);
            //遍历管辖区用户,如果是指定范围还要与指定用户取次
            if (null != respInManageVO) {
                if (!respInManageVO.getIsAdmin()){
                    for (ReportAccountRespVO respVO : respInManageVO.getList()) {
                        if (relationIds != null) {
                            if (relationIds.contains(respVO.getUserId())) {
                                accountIds.add(respVO.getUserId());
                                accountMap.put(respVO.getUserId(), respVO);
                            } else {
                                continue;
                            }
                        } else {
                            accountIds.add(respVO.getUserId());
                            accountMap.put(respVO.getUserId(), respVO);
                        }
                    }
                }else {
                    isUseAccountIds = false;
                    ReportRangeAccountReqVO reqVO1 = new ReportRangeAccountReqVO();
                    //如果是指定范围放入指定的部门或者用户id
                    if (trainingProject.getVisibleRange() == 0) {
                        isUseAccountIds = true;
                        if (CollectionUtils.isNotEmpty(relationAccountIds)) {
                            reqVO1.setAccountIds(relationAccountIds);
                        }
                        if (CollectionUtils.isNotEmpty(relationOrgIds)) {
                            reqVO1.setOrgIds(relationOrgIds);
                        }
                    }
                    reqVO1.setSiteId(siteId);
                    List<ReportAccountRespVO> accounts = reportClient.getRangeAccounts(reqVO1);
                    //将用户id拿出并将用户信息放入map中
                    if (CollectionUtils.isNotEmpty(accounts)) {
                        for (ReportAccountRespVO accountRespVO : accounts) {
                            accountIds.add(accountRespVO.getUserId());
                            accountMap.put(accountRespVO.getUserId(), accountRespVO);
                        }
                    } else {
                        return null;
                    }

                }
            } else {
                return null;
            }

        }
        //如果所有次用户不为空则进行查找考试数据
        if (CollectionUtils.isNotEmpty(accountIds)) {

            List<TrSignTime> trSignTimeList = signRecordMapper.selectSignTimeList(trainingProjectId);

            //最后的返回集
            List<ExportSignRecordVO> resultList = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(trSignTimeList)) {

                //把每个签到时间段没有记录的用户集合
                List<ExportSignRecordVO> noRecordList = new ArrayList<>();

                for (TrSignTime trSignTime : trSignTimeList) {
                    List<Long> noRecordAccountIds = new ArrayList<>();
                    noRecordAccountIds.addAll(accountIds);
                    List<ExportSignRecordVO> list = null;
                    //管理员全平台可见不需要加入用记筛选
                    if (!isUseAccountIds) {
                        list = signRecordMapper.exportSignRecord(trSignTime.getId(), null);
                    } else {
                        list = signRecordMapper.exportSignRecord(trSignTime.getId(), accountIds);
                    }
                    if (CollectionUtils.isNotEmpty(list)) {
                        for (ExportSignRecordVO recordVO : list) {
                            Long accountId1 = recordVO.getAccountId();

                            //组装用户信息
                            ReportAccountRespVO account = accountMap.get(accountId1);

                            recordVO.setTainingProjectName(trainingProject.getName());
                            recordVO.setStartTime(trSignTime.getStartTime());
                            recordVO.setEndTime(trSignTime.getEndTime());

                            if (null!=account){
                                recordVO.setAccountName(account.getUserName());
                                recordVO.setAccountFullName(account.getUserFullName());
                                recordVO.setAccountState(account.getStatus());
                                recordVO.setOrgName(account.getOrgName());
                            }

                            //如果有记录将id移除,便于之后插入没有数据的用户信息
                            if (noRecordAccountIds.contains(accountId1)) {
                                noRecordAccountIds.remove(accountId1);
                            }
                        }
                    }

                    if (CollectionUtils.isNotEmpty(list)) {
                        resultList.addAll(list);
                    }


                    //如果有没有记录的用户组装加入结果中
                    if (CollectionUtils.isNotEmpty(noRecordAccountIds)) {
                        for (Long id : noRecordAccountIds) {
                            ReportAccountRespVO accountRespVO = accountMap.get(id);
                            ExportSignRecordVO noRecordVO = new ExportSignRecordVO();
                            noRecordVO.setAccountId(id);
                            noRecordVO.setAccountName(accountRespVO.getUserName());
                            noRecordVO.setAccountFullName(accountRespVO.getUserFullName());
                            noRecordVO.setAccountState(accountRespVO.getStatus());
                            noRecordVO.setOrgName(accountRespVO.getOrgName());
                            noRecordVO.setTainingProjectName(trainingProject.getName());
                            noRecordVO.setStartTime(trSignTime.getStartTime());
                            noRecordVO.setEndTime(trSignTime.getEndTime());
                            noRecordVO.setSignState(0);
                            list.add(noRecordVO);
                            noRecordList.add(noRecordVO);
                        }
                    }
                }
                if (CollectionUtils.isNotEmpty(noRecordList)) {
                    resultList.addAll(noRecordList);
                }
                return resultList;
            } else {
                return null;
            }

        } else {
            return null;
        }
    }
    
	@Override
	public boolean insertSignRecordList(List<TrSignRecord> signRecordList) {
		if (CollectionUtils.isNotEmpty(signRecordList)) {
			for(TrSignRecord signRecord:signRecordList) {
				TrSignTime signTime = signTimeMapper.selectById(signRecord.getSignId());
				TrSignRecord trSignRecord = new TrSignRecord();
				trSignRecord.setAccountId(signRecord.getAccountId());
				trSignRecord.setSignId(signRecord.getSignId());
				trSignRecord.setType(0);
                EntityWrapper<TrSignRecord> entityWrapper = new EntityWrapper<>(trSignRecord);
                trSignRecord = this.selectOne(entityWrapper);
                if (null!=trSignRecord) {
                	//有签到记录更新入库
    				if (signRecord.getSignTime().before(signTime.getEndTime())&&signRecord.getSignTime().after(signTime.getStartTime())) {
    					signRecord.setSignStatus(0);
    				}else {
    					signRecord.setSignStatus(1);
					}
					signRecord.setId(trSignRecord.getId());
				}else {
					//没签到记录新增入库
					if (signRecord.getSignTime().before(signTime.getEndTime())&&signRecord.getSignTime().after(signTime.getStartTime())) {
    					signRecord.setSignStatus(0);
    				}else {
    					signRecord.setSignStatus(1);
					}
					signRecord.setId(idGenerator.generate());
					signRecord.setTrainingProjectId(signTime.getTrainingProjectId());
					signRecord.setCreateTime(DateUtil.now());
				}
                signTime.setUpdateTime(new Date());
                signTimeMapper.updateById(signTime);
			}
		}
			return this.insertOrUpdateBatch(signRecordList);
	}
}
